<?php

namespace Drupal\usasearch;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityManagerInterface;

/**
 * Provides a repository for Search Page config entities.
 */
// todo rename to usasearchIndexRepository
//implements UsasearchRepositoryInterface
class UsasearchRepository {
  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The search page storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $storage;

  /**
   * Constructs a new UsasearchRepository.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityManagerInterface $entity_manager
   *   The entity manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityManagerInterface $entity_manager) {
    $this->configFactory = $config_factory;
//    $this->storage = $entity_manager->getStorage('usasearch_index');
  }

  /**
   * {@inheritdoc}
   */
  public function setIndexableEntities() {
    $contentTypes = \Drupal::service('entity.manager')->getStorage('node_type')->loadMultiple();
    $api = \Drupal::service('usasearch.api');
    $enabled_types = $api->getEnabledContentTypes();
    foreach ($contentTypes as $contentType) {
      if (in_array($contentType->id(), $enabled_types)) {
        $nids = \Drupal::entityQuery('node')
          ->condition('type', $contentType->id())
          ->execute();
        $this->markReindex($nids);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getIndexableEntities() {
    $config = \Drupal::config('usasearch.settings');
    $limit = $config->get('cron_limit');

    $query = \Drupal::database()->select('digitalgovsearch');
    $query->fields('digitalgovsearch', ['nid']);
    $query->condition('reindex', 0, '!=');
    $query->range(0, (int)$limit);
    $nids = $query->execute()->fetchAll();

    return $nids;
  }

  /**
   * updates digitalgovsearch index queue
   *
   * @param $nids array
   *  an array of node ids
   *
  */
  public function markReindex($nids){

    foreach ($nids as $nid){
      $index_node = (int)usasearch_get_per_node_settings($nid);

      $query = \Drupal::database()->upsert('digitalgovsearch');
      $query->fields([
        'nid',
        'search_include',
        'reindex'
      ]);
      $query->values([
        $nid,
        $index_node,
        REQUEST_TIME,
      ]);
      $query->key('nid');
      $query->execute();
    }
  }

  /**
   * clears the digitalgovsearch index queue
   */
  //todo unused and this updates all database fields
  function clearIndexQueue(){
    \Drupal::database()->update('digitalgovsearch')
      ->condition('reindex', 0, '!=')
      ->fields([
        'reindex' => 0,
      ])
      ->execute();
  }

  /**
   * Returns an entity query instance.
   *
   * @return \Drupal\Core\Entity\Query\QueryInterface
   *   The query instance.
   */
  protected function getQuery() {
    return $this->storage->getQuery();
  }


}
